@@ -5,8 +5,12 @@ Comparison methods for comparing two RDMs objects
 """
 import numpy as np
 import scipy.stats
+from scipy import linalg
+from scipy.optimize import minimize
 from scipy.stats._stats import _kendall_dis
+from scipy.spatial.distance import squareform
 from rsatoolbox.util.matrix import pairwise_contrast_sparse
+from rsatoolbox.util.matrix import pairwise_contrast
 from rsatoolbox.util.rdm_utils import _get_n_from_reduced_vectors
 from rsatoolbox.util.rdm_utils import _get_n_from_length
 from rsatoolbox.util.matrix import row_col_indicator_g
@@ -41,6 +45,8 @@ def compare(rdm1, rdm2, method='cosine', sigma_k=None):
             'cosine_cov' = unbiased distance correlation
             which is equivalent to the cosine dinstance after whitening
 
+            'neg_riem_dist' = negative riemannian distance
+
         sigma_k (numpy.ndarray):
             covariance matrix of the pattern estimates.
             Used only for methods 'corr_cov' and 'cosine_cov'.
@@ -66,6 +72,8 @@ def compare(rdm1, rdm2, method='cosine', sigma_k=None):
         sim = compare_correlation_cov_weighted(rdm1, rdm2, sigma_k=sigma_k)
     elif method == 'cosine_cov':
         sim = compare_cosine_cov_weighted(rdm1, rdm2, sigma_k=sigma_k)
+    elif method == 'neg_riem_dist':
+        sim = compare_neg_riemannian_distance(rdm1, rdm2, sigma_k=sigma_k)
     else:
         raise ValueError('Unknown RDM comparison method requested!')
     return sim
@@ -235,7 +243,38 @@ def compare_kendall_tau_a(rdm1, rdm2):
     return sim
 
 
-def _all_combinations(vectors1, vectors2, func):
+def compare_neg_riemannian_distance(rdm1, rdm2, sigma_k=None):
+    """calculates the negative Riemannian distance between two RDMs objects.
+
+    Args:
+        rdm1 (rsatoolbox.rdm.RDMs):
+            first set of RDMs
+        rdm2 (rsatoolbox.rdm.RDMs):
+            second set of RDMs
+    Returns:
+        numpy.ndarray: dist:
+            negative Riemannian distance between the two RDMs
+    """
+    vector1, vector2, _ = _parse_input_rdms(rdm1, rdm2)
+    n_cond = _get_n_from_length(vector1.shape[1])
+    if sigma_k is None:
+        sigma_k = np.eye(n_cond)
+    P = np.block([-1*np.ones((n_cond - 1, 1)), np.eye(n_cond - 1)])
+    sigma_k_hat = P@sigma_k@P.T
+    # construct RDM to 2nd-moment (G) transformation
+    pairs = pairwise_contrast(np.arange(n_cond-1))
+    pairs[pairs == -1] = 1
+    T = np.block([
+        [np.eye(n_cond - 1), np.zeros((n_cond-1, vector1.shape[1] - n_cond + 1))],
+        [0.5 * pairs, np.diag(-0.5 * np.ones(vector1.shape[1] - n_cond + 1))]])
+    vec_G1 = vector1@np.transpose(T)
+    vec_G2 = vector2@np.transpose(T)
+
+    sim = _all_combinations(vec_G1, vec_G2, _riemannian_distance, sigma_k_hat)
+    return sim
+
+
+def _all_combinations(vectors1, vectors2, func, *args, **kwargs):
     """runs a function func on all combinations of v1 in vectors1
     and v2 in vectors2 and puts the results into an array
 
@@ -256,7 +295,7 @@ def _all_combinations(vectors1, vectors2, func):
     for v1 in vectors1:
         k2 = 0
         for v2 in vectors2:
-            value[k1, k2] = func(v1, v2)
+            value[k1, k2] = func(v1, v2, *args, **kwargs)
             k2 += 1
         k1 += 1
     return value
@@ -431,6 +470,31 @@ def _cosine(vector1, vector2):
     return cos
 
 
+def _riemannian_distance(vec_G1, vec_G2, sigma_k):
+    """computes the Riemannian distance between two vectorized second moments
+
+    Args:
+        vec_G1 (numpy.ndarray):
+            first vectorized second-moment
+        vec_G2 (numpy.ndarray):
+            second vectorized second-moment
+
+        Returns:
+            neg_riem (float):
+                negative riemannian distance
+    """
+    n_cond = _get_n_from_length(len(vec_G1))
+    G1 = np.diag(vec_G1[0:(n_cond-1)])+squareform(vec_G1[(n_cond-1):len(vec_G1)])
+    G2 = np.diag(vec_G2[0:(n_cond-1)])+squareform(vec_G2[(n_cond-1):len(vec_G2)])
+
+    def fun(theta):
+        return np.sqrt((np.log(linalg.eigvalsh(
+            np.exp(theta[0]) * G1 + np.exp(theta[1]) * sigma_k, G2))**2).sum())
+    theta = minimize(fun, (0, 0), method='Nelder-Mead')
+    neg_riem = -1 * theta.fun
+    return neg_riem
+
+
 def _kendall_tau(vector1, vector2):
     """computes the kendall-tau between two vectors
 
