@@ -546,8 +546,8 @@ def permute_rdms(rdms, p=None):
     descriptors.update({'p_inv': p_inv})
     rdm_mats = rdm_mats[:, p, :]
     rdm_mats = rdm_mats[:, :, p]
-    stims = np.array(pattern_descriptors['stim'])
-    pattern_descriptors.update({'stim': list(stims[p].astype(np.str_))})
+    stims = np.array(pattern_descriptors['index'])
+    pattern_descriptors.update({'index': list(stims[p].astype(np.str_))})
 
     rdms_p = RDMs(
         dissimilarities=rdm_mats,
